@@ -22,6 +22,12 @@
 /* Define to 'u_int32_t' if not defined anywhere. */
 #undef __u32
 
+/* Define to 'int32_t' if not defined anywhere. */
+#undef __s64
+
+/* Define to 'u_int32_t' if not defined anywhere. */
+#undef __u64
+
 /* Define to 'unsigned' if <sys/types.h> doesn't define. */
 #undef loff_t
 
