@@ -1,10 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1998-2000  Andrew Clausen, Lennert Buytenhek and Red Hat Inc.
-
-	Andrew Clausen			<clausen@alphalink.com.au>
-	Lennert Buytenhek		<buytenh@scotty.dsv.nl>
-	Matt Wilson, Red Hat Inc.	<msw@redhat.com>
+    Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,7 +27,9 @@ typedef enum {
 	PED_DEVICE_UNKNOWN	= 0,
 	PED_DEVICE_SCSI		= 1,
 	PED_DEVICE_IDE		= 2,
-	PED_DEVICE_DAC960	= 3
+	PED_DEVICE_DAC960	= 3,
+	PED_DEVICE_CPQARRAY	= 4,
+	PED_DEVICE_FILE		= 5
 } PedDeviceType;
 
 typedef struct _PedDevice PedDevice;
@@ -40,21 +38,26 @@ typedef struct _PedDevice PedDevice;
 struct _PedDevice {
 	PedDevice*	next;
 
-	char*		model;
-	char*		path;
+	char*		model;			/* description of hardware */
+	char*		path;			/* device /dev entry */
 
-	PedDeviceType	type;
+	PedDeviceType	type;			/* SCSI, IDE, etc. */
 	int		sector_size;
-	int		heads, sectors, cylinders;
-	int		geom_known;
-	short		host, did;
 	PedSector	length;
 
 	int		open_count;
+	int		read_only;
+	int		external_mode;
 	int		dirty;
-#ifndef __MSDOS__
+	int		boot_dirty;
 	int		fd;
-#endif
+
+/* x86-specific: TODO hide this in arch_specific.  But argghh!  The disk
+ * label stuff needs it, not to mention front-ends :-( */
+	int		heads, sectors, cylinders;
+	int		geom_known;
+	int		geom_already_guessed;
+	short		host, did;
 };
 
 extern void ped_device_probe_all ();
@@ -64,6 +67,10 @@ extern PedDevice* ped_device_get (const char* name);
 extern PedDevice* ped_device_get_next (const PedDevice* dev);
 extern int ped_device_open (PedDevice* dev);
 extern int ped_device_close (PedDevice* dev);
+
+extern int ped_device_begin_external_access (PedDevice* dev);
+extern int ped_device_end_external_access (PedDevice* dev);
+
 extern int ped_device_read (const PedDevice* dev, void* buffer,
 			    PedSector start, PedSector count);
 extern int ped_device_write (PedDevice* dev, const void* buffer,
