@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1998-2000 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,5 +31,8 @@ typedef struct {
 
 #define DOS_NAME	"msdos"
 
+extern void ped_disk_msdos_init ();
+extern void ped_disk_msdos_done ();
+
 #endif /* PED_DISK_DOS_H_INCLUDED */
 
