@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1998-2000 Free Software Foundation, Inc.
+    Copyright (C) 2000 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,5 +25,8 @@ typedef struct {
 
 #define LOOP_NAME	"loop"
 
+extern void ped_disk_loop_init ();
+extern void ped_disk_loop_done ();
+
 #endif /* PED_DISK_LOOP_H_INCLUDED */
 
