@@ -492,7 +492,7 @@ init_file (PedDevice* dev)
 	return 1;
 
 error:
-	return NULL;
+	return 0;
 }
 
 static int
@@ -687,6 +687,7 @@ _kernel_reread_part_table (PedDevice* dev)
 static int
 _do_refresh_open (PedDevice* dev)
 {
+	dev->open_count++;
 #ifdef linux
 	_flush_cache (dev);
 #endif
@@ -731,12 +732,14 @@ retry:
 	_flush_cache (dev);
 #endif
 
+	dev->open_count++;
 	return 1;
 }
 
 static int
 _do_refresh_close (PedDevice* dev)
 {
+	dev->open_count--;
 	return 1;
 }
 
@@ -751,6 +754,9 @@ _do_close (PedDevice* dev)
 			dev->dirty = 0;
 	}
 
+	close (dev->fd);
+	dev->open_count--;
+
 	if (dev->dirty && dev->boot_dirty && dev->type != PED_DEVICE_FILE) {
 		/* ouch! */
 		ped_exception_throw (
@@ -766,7 +772,6 @@ _do_close (PedDevice* dev)
 			  "loader from the rescue disk.  Read section 4 of "
 			  "the Parted User documentation for more "
 			  "information."));
-		close (dev->fd);
 		return 1;
 	}
 
@@ -780,6 +785,7 @@ _do_close (PedDevice* dev)
 			  "reboot your computer before doing anything with "
 			  "%s."),
 			dev->path, strerror (errno), dev->path);
+		return 1;
 	}
 
 	if (dev->boot_dirty && dev->type != PED_DEVICE_FILE) {
@@ -789,10 +795,10 @@ _do_close (PedDevice* dev)
 			_("You should reinstall your boot loader before "
 			  "rebooting.  Read section 4 of the Parted User "
 			  "documentation for more information."));
+		return 1;
 	}
 #endif /* linux */
 
-	close (dev->fd);
 	return 1;
 }
 
@@ -802,10 +808,10 @@ ped_device_open (PedDevice* dev)
 	PED_ASSERT (dev != NULL, return 0);
 	PED_ASSERT (!dev->external_mode, return 0);
 
-	if (dev->open_count++)
+	if (dev->open_count)
 		return _do_refresh_open (dev);
-
-	return _do_open (dev);
+	else
+		return _do_open (dev);
 }
 
 int
@@ -813,12 +819,12 @@ ped_device_close (PedDevice* dev)
 {
 	PED_ASSERT (dev != NULL, return 0);
 	PED_ASSERT (!dev->external_mode, return 0);
+	PED_ASSERT (dev->open_count > 0, return 0);
 
-	if (--dev->open_count)
+	if (dev->open_count > 1)
 		return _do_refresh_close (dev);
-
-	return _do_close (dev);
-
+	else
+		return _do_close (dev);
 }
 
 int
