@@ -630,7 +630,7 @@ ped_partition_new (const PedDisk* disk, PedPartitionType type,
 
 	if (fs_type || part->type == PED_PARTITION_EXTENDED) {
 		if (!ped_partition_set_system (part, fs_type))
-			goto error;
+			goto error_destroy_part;
 	}
 	return part;
 
@@ -763,14 +763,16 @@ _fs_set_system (const PedFileSystemType* fs_type, PedFileSystem* fs,
 int
 ped_partition_set_system (PedPartition* part, const PedFileSystemType* fs_type)
 {
-	PedDiskType*		disk_type;
 	PedFileSystem*		fs;
 
 	PED_ASSERT (part != NULL, return 0);
+	PED_ASSERT (ped_partition_is_active (part), return 0);
 
 	part->fs_type = fs_type;
 
 	if (part->type & PED_PARTITION_EXTENDED) {
+		PedDiskType*		disk_type;
+
 		PED_ASSERT (part->geom.disk != NULL, return 0);
 		disk_type = part->geom.disk->type;
 		PED_ASSERT (disk_type != NULL, return 0);
@@ -778,10 +780,9 @@ ped_partition_set_system (PedPartition* part, const PedFileSystemType* fs_type)
 		PED_ASSERT (disk_type->ops->partition_set_extended_system
 				!= NULL,
 			    return 0);
-	}
 
-	if (part->type & PED_PARTITION_EXTENDED)
 		return disk_type->ops->partition_set_extended_system (part);
+	}
 
 	PED_ASSERT (fs_type != NULL, return 0);
 	PED_ASSERT (fs_type->ops->set_system != NULL, return 0);
@@ -833,8 +834,10 @@ ped_partition_set_name (PedPartition* part, const char* name)
 	PedDisk*	disk;
 
 	PED_ASSERT (part != NULL, return 0);
+	PED_ASSERT (part->geom.disk != NULL, return 0);
+	PED_ASSERT (ped_partition_is_active (part), return 0);
+
 	disk = part->geom.disk;
-	PED_ASSERT (disk != NULL, return 0);
 	if (!_assert_partition_name_feature (disk->type))
 		return 0;
 	PED_ASSERT (disk->type->ops->partition_set_name != NULL, return 0);
@@ -849,8 +852,10 @@ ped_partition_get_name (const PedPartition* part)
 	PedDisk*	disk;
 
 	PED_ASSERT (part != NULL, return NULL);
+	PED_ASSERT (part->geom.disk != NULL, return NULL);
+	PED_ASSERT (ped_partition_is_active (part), return NULL);
+
 	disk = part->geom.disk;
-	PED_ASSERT (disk != NULL, return NULL);
 	if (!_assert_partition_name_feature (disk->type))
 		return NULL;
 	PED_ASSERT (disk->type->ops->partition_get_name != NULL, return NULL);
@@ -1062,7 +1067,8 @@ _disk_check_sanity (PedDisk* disk)
 		if (!strcasecmp (disk->type->name, "msdos")
 			&& disk->dev->geom_known
 			&& walk->type & PED_PARTITION_FREESPACE)
-			PED_ASSERT (walk->geom.length > 63 * 5, return 0);
+			PED_ASSERT (walk->geom.length > 5 * disk->dev->sectors,
+				    return 0);
 	}
 
 	if (!ped_disk_extended_partition (disk))
