@@ -159,6 +159,8 @@ ped_disk_msdos_done ()
 	ped_unregister_disk_type (&msdos_disk_type);
 }
 
+static int raw_part_is_lba (const DosRawPartition* raw_part);
+
 /* sanity check for msdos_probe()
  *
  * We have to be very lenient, because the BIOS geometry may have been
@@ -183,13 +185,22 @@ check_partition_sanity (const PedDevice* dev, const DosRawPartition* raw_part)
 		return 0;
 	if (memcmp (&raw_part->chs_end, "\0\0\0", 3) == 0)
 		return 0;
+	if (!raw_part->start || !raw_part->length)
+		return 0;
+
+	/* somewhat subtle.  Windows ME generates broken partition tables.
+	 * Everyone agreed to have partitions > 1024 cylinder have the end
+	 * marked as cylinder 1023.  Windows ME, however, uses (end cyl % 1024)
+	 * However, these partitions are always marked LBA.  (A flag that
+	 * is only available for FAT partitions).  Hence, we should not do
+	 * the normal sanity checks, if the partition is marked as LBA.
+	 */
+	if (raw_part_is_lba (raw_part))
+		return 1;
 
 	legacy_start = chs_to_sector (dev, &raw_part->chs_start);
 	legacy_end = chs_to_sector (dev, &raw_part->chs_end);
 
-	if (!raw_part->start || !raw_part->length)
-		return 0;
-
 	max_chs_sector = (dev->sector_size / 512) * 1024 * 255 * 63;
 	if (legacy_start
 	    && PED_LE32_TO_CPU (raw_part->start) > max_chs_sector)
@@ -545,7 +556,9 @@ process_inconsistent_table (PedDisk* disk, const DosRawPartition* raw_part,
 			"There are many reasons why this might be the case.  "
 			"Often, the reason is that Linux detected the BIOS "
 			"geometry incorrectly.  However, this does not "
-			"appear to be the case here."),
+			"appear to be the case here.  It is safe to ignore,"
+			"but ignoring may cause (fixable) problems with "
+			"some boot loaders."),
 			disk->dev->path)
 				== PED_EXCEPTION_CANCEL)
 			return 0;
@@ -596,7 +609,7 @@ raw_part_is_hidden (DosRawPartition* raw_part)
 }
 
 static int
-raw_part_is_lba (DosRawPartition* raw_part)
+raw_part_is_lba (const DosRawPartition* raw_part)
 {
 	PED_ASSERT (raw_part != NULL, return 0);
 
@@ -834,8 +847,7 @@ write_ext_table (PedDisk* disk, PedSector sector, PedPartition* logical)
 	part = ped_disk_get_partition (disk, logical->num + 1);
 	if (part) {
 		geom = ped_geometry_new (disk, part->prev->geom.start,
-				ped_disk_extended_partition (disk)->geom.end
-				- part->prev->geom.start + 1);
+				part->geom.end - part->prev->geom.start + 1);
 		if (!geom)
 			return 0;
 		fill_ext_raw_part_geom (&table.partitions[1], geom, lba_offset);
@@ -1380,7 +1392,7 @@ _align_logical (PedPartition* part, const PedConstraint* constraint)
 	start_base = ped_round_down_to (part->geom.start, cyl_size);
 
 	for (head = _logical_min_start_head (part, ext_part, 0);
-	     head < 5; head++) {
+	     head < PED_MIN (5, dev->heads); head++) {
 		PedConstraint*	disk_constraint;
 		PedSector	start = start_base + head * dev->sectors;
 
@@ -1406,6 +1418,15 @@ _align_logical (PedPartition* part, const PedConstraint* constraint)
 	return 0;
 }
 
+static int
+_align (PedPartition* part, const PedConstraint* constraint)
+{
+	if (part->type == PED_PARTITION_LOGICAL)
+		return _align_logical (part, constraint);
+	else
+		return _align_primary (part, constraint);
+}
+
 static PedConstraint*
 _no_geom_constraint (PedDisk* disk, PedSector start, PedSector end)
 {
@@ -1507,43 +1528,40 @@ _align_logical_no_geom (PedPartition* part, const PedConstraint* constraint)
 	return 0;
 }
 
+static int
+_align_no_geom (PedPartition* part, const PedConstraint* constraint)
+{
+	if (part->type == PED_PARTITION_LOGICAL)
+		return _align_logical_no_geom (part, constraint);
+	else
+		return _align_primary_no_geom (part, constraint);
+}
+
 static int
 msdos_partition_align (PedPartition* part, const PedConstraint* constraint)
 {
-	int	status;
-
 	PED_ASSERT (part != NULL, return 0);
 
-	if (part->geom.disk->dev->geom_known) {
-		if (part->type == PED_PARTITION_PRIMARY
-		    || part->type == PED_PARTITION_EXTENDED) {
-			if (_align_primary (part, constraint))
-				return 1;
-		} else {
-			if (_align_logical (part, constraint))
-				return 1;
-		}
+	if (_align (part, constraint))
+		return 1;
 
+	if (part->geom.disk->dev->geom_known) {
 		if (ped_exception_throw (
 			PED_EXCEPTION_WARNING,
 			PED_EXCEPTION_IGNORE_CANCEL,
 			_("Unable to align partition properly.  This probably "
 			  "means that another partitioning tool generated an "
 			  "incorrect partition table, because it didn't have "
-			  "the correct BIOS geometry."))
-				!= PED_EXCEPTION_IGNORE)
+			  "the correct BIOS geometry.  It is safe to ignore,"
+			  "but ignoring may cause (fixable) problems with "
+			  "some boot loaders."))
+				== PED_EXCEPTION_CANCEL)
 			return 0;
 		part->geom.disk->dev->geom_known = 0;
 	}
 
-	if (part->type == PED_PARTITION_PRIMARY
-	    || part->type == PED_PARTITION_EXTENDED) {
-		if (_align_primary_no_geom (part, constraint))
-			return 1;
-	} else {
-		if (_align_logical_no_geom (part, constraint))
-			return 1;
-	}
+	if (_align_no_geom (part, constraint))
+		return 1;
 
 	ped_exception_throw (
 		PED_EXCEPTION_ERROR,
