@@ -279,9 +279,11 @@ error:
 static int
 mac_free (PedDisk* disk)
 {
+	MacDiskData*	mac_disk_data = disk->disk_specific;
+
 	ped_device_close (disk->dev);
-	ped_free (disk->disk_specific);
 	ped_disk_free (disk);
+	ped_free (mac_disk_data);
 	return 1;
 }
 
@@ -1130,6 +1132,7 @@ mac_partition_set_flag (PedPartition* part, PedPartitionFlag flag, int state)
 		} else {
 			strcpy (mac_data->system_name, "Apple_Bootstrap");
 			mac_data->status = 0x33;
+			status = 1;
 		}
 		return status;
 
