@@ -80,7 +80,7 @@ ped_file_system_type_get (const char* name)
 }
 
 PedFileSystemType*
-ped_file_system_type_get_next (PedFileSystemType* fs_type)
+ped_file_system_type_get_next (const PedFileSystemType* fs_type)
 {
 	if (fs_type)
 		return fs_type->next;
