@@ -1,7 +1,7 @@
 /*
     ext2.c -- generic ext2 stuff
-    Copyright (C) 1998,99 Lennert Buytenhek <buytenh@gnu.org>
-
+    Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+  
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
@@ -51,11 +51,11 @@ int ext2_get_block_state(struct ext2_fs *fs, blk_t block)
 	int offset;
 	int state;
 
-	block -= fs->sb.s_first_data_block;
-	group = block / fs->sb.s_blocks_per_group;
-	offset = block % fs->sb.s_blocks_per_group;
+	block -= EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb);
+	group = block / EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb);
+	offset = block % EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb);
 
-	bh = ext2_bread(fs, fs->gd[group].bg_block_bitmap);
+	bh = ext2_bread(fs, EXT2_GROUP_BLOCK_BITMAP(fs->gd[group]));
 	state = bh->data[offset>>3] & _bitmap[offset&7];
 	ext2_brelse(bh, 0);
 
@@ -67,13 +67,16 @@ blk_t ext2_find_free_block(struct ext2_fs *fs)
 	int i;
 
 	for (i=0;i<fs->numgroups;i++)
-		if (fs->gd[i].bg_free_blocks_count)
+		if (EXT2_GROUP_FREE_BLOCKS_COUNT(fs->gd[i]))
 		{
 			blk_t j;
 			blk_t offset;
 
-			offset = i * fs->sb.s_blocks_per_group + fs->sb.s_first_data_block;
-			for (j=fs->adminblocks;j<fs->sb.s_blocks_per_group;j++)
+			offset = i * EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb)
+				 + EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb);
+			for (j=fs->adminblocks;
+			     j<EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb);
+			     j++)
 				if (ext2_is_data_block(fs, offset + j) &&
 				    !ext2_get_block_state(fs, offset + j))
 					return offset + j;
@@ -93,13 +96,13 @@ ino_t ext2_find_free_inode(struct ext2_fs *fs)
 	int i;
 
 	for (i=0;i<fs->numgroups;i++)
-		if (fs->gd[i].bg_free_inodes_count)
+		if (EXT2_GROUP_FREE_INODES_COUNT(fs->gd[i]))
 		{
 			ino_t j;
 			ino_t offset;
 
-			offset = i * fs->sb.s_inodes_per_group + 1;
-			for (j=0;j<fs->sb.s_inodes_per_group;j++)
+			offset = i * EXT2_SUPER_INODES_PER_GROUP(fs->sb) + 1;
+			for (j=0;j<EXT2_SUPER_INODES_PER_GROUP(fs->sb);j++)
 				if (!ext2_get_inode_state(fs, offset + j))
 					return offset + j;
 
@@ -161,11 +164,11 @@ int ext2_set_block_state(struct ext2_fs *fs, blk_t block, int state, int updatem
 	int                      group;
 	int                      offset;
 
-	block -= fs->sb.s_first_data_block;
-	group = block/fs->sb.s_blocks_per_group;
-	offset = block%fs->sb.s_blocks_per_group;
+	block -= EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb);
+	group = block / EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb);
+	offset = block % EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb);
 
-	bh = ext2_bread(fs, fs->gd[group].bg_block_bitmap);
+	bh = ext2_bread(fs, EXT2_GROUP_BLOCK_BITMAP(fs->gd[group]));
 	bh->dirty = 1;
 	if (state)
 		bh->data[offset>>3] |= _bitmap[offset&7];
@@ -179,8 +182,10 @@ int ext2_set_block_state(struct ext2_fs *fs, blk_t block, int state, int updatem
 
 		diff = state ? -1 : 1;
 
-		fs->gd[group].bg_free_blocks_count += diff;
-		fs->sb.s_free_blocks_count += diff;
+		fs->gd[group].bg_free_blocks_count = PED_CPU_TO_LE16
+			(EXT2_GROUP_FREE_BLOCKS_COUNT(fs->gd[group]) + diff);
+		fs->sb.s_free_blocks_count = PED_CPU_TO_LE32
+			(EXT2_SUPER_FREE_BLOCKS_COUNT(fs->sb) + diff);
 		fs->metadirty |= EXT2_META_SB | EXT2_META_GD;
 	}
 	return 1;
@@ -279,10 +284,12 @@ off_t ext2_get_inode_offset(struct ext2_fs *fs, ino_t inode, blk_t *block)
 
 	inode--;
 
-	group = inode / fs->sb.s_inodes_per_group;
-	offset = (inode % fs->sb.s_inodes_per_group) * sizeof(struct ext2_inode);
+	group = inode / EXT2_SUPER_INODES_PER_GROUP(fs->sb);
+	offset = (inode % EXT2_SUPER_INODES_PER_GROUP(fs->sb))
+	       	 * sizeof(struct ext2_inode);
 
-	*block = fs->gd[group].bg_inode_table + (offset >> fs->logsize);
+	*block = EXT2_GROUP_INODE_TABLE(fs->gd[group])
+	         + (offset >> fs->logsize);
 
 	return offset & (fs->blocksize - 1);
 }
@@ -295,10 +302,10 @@ int ext2_get_inode_state(struct ext2_fs *fs, ino_t inode)
 	int                      ret;
 
 	inode--;
-	group = inode / fs->sb.s_inodes_per_group;
-	offset = inode % fs->sb.s_inodes_per_group;
+	group = inode / EXT2_SUPER_INODES_PER_GROUP(fs->sb);
+	offset = inode % EXT2_SUPER_INODES_PER_GROUP(fs->sb);
 
-	bh = ext2_bread(fs, fs->gd[group].bg_inode_bitmap);
+	bh = ext2_bread(fs, EXT2_GROUP_INODE_BITMAP(fs->gd[group]));
 	ret = bh->data[offset>>3] & _bitmap[offset&7];
 	ext2_brelse(bh, 0);
 
@@ -329,10 +336,10 @@ int ext2_set_inode_state(struct ext2_fs *fs, ino_t inode, int state, int updatem
 	int                      offset;
 
 	inode--;
-	group = inode / fs->sb.s_inodes_per_group;
-	offset = inode % fs->sb.s_inodes_per_group;
+	group = inode / EXT2_SUPER_INODES_PER_GROUP(fs->sb);
+	offset = inode % EXT2_SUPER_INODES_PER_GROUP(fs->sb);
 
-	bh = ext2_bread(fs, fs->gd[group].bg_inode_bitmap);
+	bh = ext2_bread(fs, EXT2_GROUP_INODE_BITMAP(fs->gd[group]));
 	if (!bh)
 		return 0;
 	bh->dirty = 1;
@@ -348,100 +355,108 @@ int ext2_set_inode_state(struct ext2_fs *fs, ino_t inode, int state, int updatem
 
 		diff = state ? -1 : 1;
 
-		fs->gd[group].bg_free_inodes_count += diff;
-		fs->sb.s_free_inodes_count += diff;
+		fs->gd[group].bg_free_inodes_count = PED_CPU_TO_LE16
+			(EXT2_GROUP_FREE_INODES_COUNT(fs->gd[group]) + diff);
+		fs->sb.s_free_inodes_count = PED_CPU_TO_LE32
+			(EXT2_SUPER_FREE_INODES_COUNT(fs->sb) + diff);
 		fs->metadirty = EXT2_META_SB | EXT2_META_GD;
 	}
 	return 1;
 }
 
+static void
+_inode_update_size(struct ext2_fs *fs, struct ext2_inode *inode, int delta)
+{
+	int		 i512perblock = 1 << (fs->logsize - 9);
+
+	/* i_blocks is in 512 byte blocks */
+	inode->i_blocks = PED_CPU_TO_LE32(EXT2_INODE_BLOCKS(*inode)
+		       			  + delta * i512perblock);
+	inode->i_size = PED_CPU_TO_LE32(EXT2_INODE_SIZE(*inode)
+		       			+ delta * fs->blocksize);
+	inode->i_mtime = PED_CPU_TO_LE32(time(NULL));
+}
+
 int ext2_do_inode(struct ext2_fs *fs, struct ext2_inode *inode, blk_t block,
 		   int action)
 {
 	struct ext2_buffer_head *bh;
 	__u32			*udata;
-	int			 count = 0;
+	blk_t			 count = 0;
 	int			 i;
 	int			 u32perblock = fs->blocksize >> 2;
 	int			 i512perblock = 1 << (fs->logsize - 9);
 
-	if (block == 0 || inode->i_mode == 0)
+	if (block == 0 || EXT2_INODE_MODE(*inode) == 0)
 		return -1;
 
 	if (fs->opt_debug)
 		switch (action)
 		{
 			case EXT2_ACTION_ADD:
-				fprintf(stderr,"adding 0x%04x to inode\n",block);
+				fprintf(stderr,"adding 0x%04x to inode\n",
+					block);
 				break;
 			case EXT2_ACTION_DELETE:
 				fprintf(stderr,"deleting 0x%04x from inode\n",
 					block);
 				break;
 			case EXT2_ACTION_FIND:
-				fprintf(stderr,"finding 0x%04x in inode\n",block);
+				fprintf(stderr,"finding 0x%04x in inode\n",
+					block);
 				break;
 		}
 
 	/* Direct blocks for first 12 blocks */
 	for (i = 0; i < EXT2_NDIR_BLOCKS; i++)
 	{
-		if (action == EXT2_ACTION_ADD && !inode->i_block[i])
+		if (action == EXT2_ACTION_ADD && !EXT2_INODE_BLOCK(*inode, i))
 		{
-			inode->i_block[i] = block;
-			/* i_blocks is in 512 byte blocks */
-			inode->i_blocks += i512perblock;
-			inode->i_size += fs->blocksize;
-			inode->i_mtime = time(NULL);
+			inode->i_block[i] = PED_CPU_TO_LE32(block);
+			_inode_update_size (fs, inode, 1);
 			ext2_set_block_state(fs, block, 1, 1);
 			return i;
 		}
-		if (inode->i_block[i] == block)
+		if (EXT2_INODE_BLOCK(*inode, i) == block)
 		{
 			if (action == EXT2_ACTION_DELETE)
 			{
 				inode->i_block[i] = 0;
-				inode->i_blocks -= i512perblock;
-				inode->i_size -= fs->blocksize;
-				inode->i_mtime = time(NULL);
+				_inode_update_size (fs, inode, -1);
 				ext2_set_block_state(fs, block, 0, 1);
 			}
 			return i;
 		}
-		if (inode->i_block[i])
+		if (EXT2_INODE_BLOCK(*inode, i))
 			count += i512perblock;
 	}
 
-	count += inode->i_block[EXT2_IND_BLOCK] ? i512perblock : 0;
-	count += inode->i_block[EXT2_DIND_BLOCK] ? i512perblock : 0;
-	count += inode->i_block[EXT2_TIND_BLOCK] ? i512perblock : 0;
+	count += EXT2_INODE_BLOCK(*inode, EXT2_IND_BLOCK) ? i512perblock : 0;
+	count += EXT2_INODE_BLOCK(*inode, EXT2_DIND_BLOCK) ? i512perblock : 0;
+	count += EXT2_INODE_BLOCK(*inode, EXT2_TIND_BLOCK) ? i512perblock : 0;
 
-	if (!inode->i_block[EXT2_IND_BLOCK] ||
-	    (count >= inode->i_blocks && action != EXT2_ACTION_ADD))
+	if (!EXT2_INODE_BLOCK(*inode, EXT2_IND_BLOCK) ||
+	    (count >= EXT2_INODE_BLOCKS(*inode) && action != EXT2_ACTION_ADD))
 		return -1;
 
-	bh = ext2_bread(fs, inode->i_block[EXT2_IND_BLOCK]);
+	bh = ext2_bread(fs, EXT2_INODE_BLOCK(*inode, EXT2_IND_BLOCK));
 	udata = (__u32 *)bh->data;
 
 	/* Indirect blocks for next 256/512/1024 blocks (for 1k/2k/4k blocks) */
 	for (i = 0; i < u32perblock; i++) {
 		if (action == EXT2_ACTION_ADD && !udata[i]) {
 			bh->dirty = 1;
-			udata[i] = block;
-			inode->i_blocks += i512perblock;
-			inode->i_size += fs->blocksize;
-			inode->i_mtime = time(NULL);
+			udata[i] = PED_CPU_TO_LE32(block);
+			_inode_update_size (fs, inode, 1);
 			ext2_set_block_state(fs, block, 1, 1);
 			ext2_brelse(bh, 0);
 			return EXT2_NDIR_BLOCKS + i;
 		}
-		if (udata[i] == block) {
+		if (PED_LE32_TO_CPU(udata[i]) == block) {
 			if (action == EXT2_ACTION_DELETE) {
 				bh->dirty = 1;
 				udata[i] = 0;
-				inode->i_blocks -= i512perblock;
-				inode->i_size -= fs->blocksize;
-				inode->i_mtime = time(NULL);
+				_inode_update_size (fs, inode, -1);
 				ext2_set_block_state(fs, block, 0, 1);
 			}
 			ext2_brelse(bh, 0);
@@ -450,7 +465,7 @@ int ext2_do_inode(struct ext2_fs *fs, struct ext2_inode *inode, blk_t block,
 		if (udata[i])
 		{
 			count += i512perblock;
-			if (count >= inode->i_blocks &&
+			if (count >= EXT2_INODE_BLOCKS(*inode) &&
 			    action != EXT2_ACTION_ADD)
 				return -1;
 		}
@@ -458,10 +473,10 @@ int ext2_do_inode(struct ext2_fs *fs, struct ext2_inode *inode, blk_t block,
 
 	ext2_brelse(bh, 0);
 
-	if (!inode->i_block[EXT2_DIND_BLOCK] ||
-	    (count >= inode->i_blocks && action != EXT2_ACTION_ADD))
+	if (!EXT2_INODE_BLOCK(*inode, EXT2_DIND_BLOCK) ||
+	    (count >= EXT2_INODE_BLOCKS(*inode) && action != EXT2_ACTION_ADD))
 		return -1;
-	bh = ext2_bread(fs, inode->i_block[EXT2_DIND_BLOCK]);
+	bh = ext2_bread(fs, EXT2_INODE_BLOCK(*inode, EXT2_DIND_BLOCK));
 	udata = (__u32 *)bh->data;
 
 	/* Double indirect blocks for next 2^16/2^18/2^20 1k/2k/4k blocks */
@@ -474,29 +489,25 @@ int ext2_do_inode(struct ext2_fs *fs, struct ext2_inode *inode, blk_t block,
 			ext2_brelse(bh, 0);
 			return -1;
 		}
-		bh2 = ext2_bread(fs, udata[i]);
+		bh2 = ext2_bread(fs, PED_LE32_TO_CPU(udata[i]));
 		udata2 = (__u32 *)bh2->data;
 		count += i512perblock;
 
 		for (j = 0; j < u32perblock; j++) {
 			if (action == EXT2_ACTION_ADD && !udata2[j]) {
 				bh2->dirty = 1;
-				udata2[j] = block;
-				inode->i_blocks += i512perblock;
-				inode->i_size += fs->blocksize;
-				inode->i_mtime = time(NULL);
+				udata2[j] = PED_CPU_TO_LE32(block);
+				_inode_update_size (fs, inode, 1);
 				ext2_set_block_state(fs, block, 1, 1);
 				ext2_brelse(bh, 0);
 				ext2_brelse(bh2, 0);
 				return EXT2_NDIR_BLOCKS + i * u32perblock + j;
 			}
-			if (udata2[j] == block) {
+			if (PED_LE32_TO_CPU(udata2[j]) == block) {
 				if (action == EXT2_ACTION_DELETE) {
 					bh2->dirty = 1;
 					udata2[j] = 0;
-					inode->i_blocks -= i512perblock;
-					inode->i_size -= fs->blocksize;
-					inode->i_mtime = time(NULL);
+					_inode_update_size (fs, inode, -1);
 					ext2_set_block_state(fs, block, 0, 1);
 				}
 				ext2_brelse(bh, 0);
@@ -506,7 +517,7 @@ int ext2_do_inode(struct ext2_fs *fs, struct ext2_inode *inode, blk_t block,
 			if (udata2[j])
 			{
 				count += i512perblock;
-				if (count >= inode->i_blocks &&
+				if (count >= EXT2_INODE_BLOCKS(*inode) &&
 				    action != EXT2_ACTION_ADD)
 					return -1;
 			}
@@ -620,20 +631,22 @@ int ext2_commit_metadata(struct ext2_fs *fs, int copies)
 	if (wmeta == EXT2_META_CLEAN)
 		return 1;
 
-	fs->sb.s_r_blocks_count = (fs->r_frac * (loff_t)fs->sb.s_blocks_count)
-				  / 100;
+	fs->sb.s_r_blocks_count = PED_CPU_TO_LE32 (
+		fs->r_frac * (loff_t)EXT2_SUPER_BLOCKS_COUNT(fs->sb)
+				  / 100);
 
-	memset(sb, 0, fs->blocksize);
+	if (!ext2_read_blocks (fs, sb, 0, 1))
+		return 0;
 
-	if (fs->sb.s_first_data_block)
+	if (EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb))
 		memcpy(sb, &fs->sb, 1024);
 	else
 		memcpy(sb+1024, &fs->sb, 1024);
 
 	num = copies & EXT2_META_BACKUP ? fs->numgroups : 1;
 
-	for (i = 0, sb_block = fs->sb.s_first_data_block; i < num;
-	     i++, sb_block += fs->sb.s_blocks_per_group)
+	for (i = 0, sb_block = EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb); i < num;
+	     i++, sb_block += EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb))
 	{
 
 		if (!ext2_is_group_sparse(fs, i))
@@ -669,7 +682,8 @@ static int ext2_determine_itoffset(struct ext2_fs *fs)
 {
 	int i;
 
-	fs->itoffset = fs->gd[0].bg_inode_table - fs->sb.s_first_data_block;
+	fs->itoffset = EXT2_GROUP_INODE_TABLE(fs->gd[0])
+		       - EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb);
 
 	for (i=0;i<fs->numgroups;i++)
 	{
@@ -678,8 +692,8 @@ static int ext2_determine_itoffset(struct ext2_fs *fs)
 		blk_t ib;
 		blk_t it;
 
-		start = fs->sb.s_first_data_block
-			+ (i * fs->sb.s_blocks_per_group);
+		start = EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb)
+			+ (i * EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb));
 		it = start + fs->itoffset;
 
 		if (ext2_is_group_sparse(fs, i))
@@ -693,11 +707,11 @@ static int ext2_determine_itoffset(struct ext2_fs *fs)
 			ib = start + 1;
 		}
 
-		if (fs->gd[i].bg_block_bitmap != bb ||
-		    fs->gd[i].bg_inode_bitmap != ib ||
-		    fs->gd[i].bg_inode_table != it)
+		if (EXT2_GROUP_BLOCK_BITMAP(fs->gd[i]) != bb ||
+		    EXT2_GROUP_INODE_BITMAP(fs->gd[i]) != ib ||
+		    EXT2_GROUP_INODE_TABLE(fs->gd[i]) != it)
 		{
-			ped_exception_throw (PED_EXCEPTION_BUG,
+			ped_exception_throw (PED_EXCEPTION_NO_FEATURE,
 				PED_EXCEPTION_CANCEL,
 			_("This ext2 filesystem has a rather strange layout!  "
 			  "Please use dumpe2fs on this filesystem and send it "
@@ -728,7 +742,7 @@ struct ext2_fs *ext2_open(struct ext2_dev_handle *handle, int state)
 	handle->ops->set_blocksize(handle->cookie, 10);
 
 	if (!handle->ops->read(handle->cookie, &fs->sb, 1, 1)
-	    || fs->sb.s_magic != EXT2_SUPER_MAGIC)
+	    || EXT2_SUPER_MAGIC(fs->sb) != EXT2_SUPER_MAGIC_CONST)
 	{
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
 			_("Invalid superblock.  Are you sure this is an ext2 "
@@ -741,62 +755,72 @@ struct ext2_fs *ext2_open(struct ext2_dev_handle *handle, int state)
 	fs->opt_safe = 1;
 	fs->opt_verbose = 0;
 
-	if (fs->sb.s_state & EXT2_ERROR_FS & ~(state & EXT2_ERROR_FS))
+	if (EXT2_SUPER_STATE(fs->sb) & EXT2_ERROR_FS & ~(state & EXT2_ERROR_FS))
 	{
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
 		     _("Filesystem has errors!  Run e2fsck first!"));
-		goto error_free_gd;
+		goto error_free_fs;
 	}
 
-	if (!((fs->sb.s_state | state) & EXT2_VALID_FS))
+	if (!((EXT2_SUPER_STATE(fs->sb) | state) & EXT2_VALID_FS))
 	{
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
 		     _("Filesystem was not cleanly unmounted!  "
 		       "Run e2fsck first!"));
-		goto error_free_gd;
+		goto error_free_fs;
 	}
 
-	if (fs->sb.s_feature_compat ||
-	    (fs->sb.s_feature_incompat & ~EXT2_FEATURE_INCOMPAT_FILETYPE) ||
-	    (fs->sb.s_feature_ro_compat & ~EXT2_FEATURE_RO_COMPAT_SPARSE_SUPER))
+	if (EXT2_SUPER_FEATURE_COMPAT(fs->sb) ||
+	    (EXT2_SUPER_FEATURE_INCOMPAT(fs->sb)
+	    		& ~EXT2_FEATURE_INCOMPAT_FILETYPE) ||
+	    (EXT2_SUPER_FEATURE_RO_COMPAT(fs->sb)
+	     		& ~EXT2_FEATURE_RO_COMPAT_SPARSE_SUPER))
 	{
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
 		     _("Filesystem has incompatible feature enabled"));
-		goto error_free_gd;
+		goto error_free_fs;
 	}
 
 	fs->devhandle = handle;
-	fs->logsize = fs->sb.s_log_block_size + 10;
+	fs->logsize = EXT2_SUPER_LOG_BLOCK_SIZE(fs->sb) + 10;
 	handle->ops->set_blocksize(handle->cookie, fs->logsize);
 
 	if (!ext2_bcache_init(fs))
 	{
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
 				     _("Error allocating buffer cache."));
-		goto error_free_gd;
+		goto error_free_fs;
 	}
 
 	fs->blocksize = 1 << fs->logsize;
 
-	fs->numgroups = howmany(fs->sb.s_blocks_count - fs->sb.s_first_data_block, fs->sb.s_blocks_per_group);
-	fs->gdblocks = howmany(fs->numgroups * sizeof(struct ext2_group_desc), fs->blocksize);
-	fs->inodeblocks = howmany(fs->sb.s_inodes_per_group * sizeof(struct ext2_inode), fs->blocksize);
-	fs->r_frac = howmany(100 * (loff_t)fs->sb.s_r_blocks_count, fs->sb.s_blocks_count);
+	fs->numgroups = howmany(EXT2_SUPER_BLOCKS_COUNT(fs->sb)
+		       		- EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb),
+				EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb));
+	fs->gdblocks = howmany(fs->numgroups * sizeof(struct ext2_group_desc),
+			       fs->blocksize);
+	fs->inodeblocks = howmany(EXT2_SUPER_INODES_PER_GROUP(fs->sb)
+		       		  * sizeof(struct ext2_inode),
+				  fs->blocksize);
+	fs->r_frac = howmany(100 * (loff_t)EXT2_SUPER_R_BLOCKS_COUNT(fs->sb),
+		       	     EXT2_SUPER_BLOCKS_COUNT(fs->sb));
 	fs->adminblocks = 3 + fs->gdblocks + fs->inodeblocks;
 
 	fs->sparse = 0;
-	if (fs->sb.s_feature_ro_compat & EXT2_FEATURE_RO_COMPAT_SPARSE_SUPER)
+	if (EXT2_SUPER_FEATURE_RO_COMPAT(fs->sb)
+			& EXT2_FEATURE_RO_COMPAT_SPARSE_SUPER)
 		fs->sparse = 1;
 
 	fs->gd = ped_malloc (fs->numgroups * sizeof (struct ext2_group_desc)
 			     	+ fs->blocksize);
 	if (!fs->gd)
-		goto error_free_fs;
+		goto error_deinit_bcache;
 
-	ext2_read_blocks(fs, fs->gd, fs->sb.s_first_data_block + 1, fs->gdblocks);
+	ext2_read_blocks(fs, fs->gd, EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb) + 1,
+			 fs->gdblocks);
 
 	if (!ext2_determine_itoffset(fs))
-		goto error_deinit_bcache;
+		goto error_free_gd;
 
 	fs->metadirty = 0;
 
@@ -806,10 +830,10 @@ struct ext2_fs *ext2_open(struct ext2_dev_handle *handle, int state)
 
 	return fs;
 
-error_deinit_bcache:
-	ext2_bcache_deinit(fs);
 error_free_gd:
 	ped_free(fs->gd);
+error_deinit_bcache:
+	ext2_bcache_deinit(fs);
 error_free_fs:
 	ped_free(fs);
 error:
