@@ -1,7 +1,7 @@
 /*
     ext2_block_relocator.c -- ext2 block relocator
-    Copyright (C) 1998,99 Lennert Buytenhek <buytenh@gnu.org>
-
+    Copyright (C) 1998-2000 Free Software Foundation, Inc.
+  
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
@@ -187,7 +187,8 @@ static int doindblock(struct ext2_fs *fs,
 
 	for (i=0;i<(fs->blocksize >> 2);i++)
 		if (uptr[i])
-			if (!doblock(fs, state, uptr[i], blk, i<<2, 0))
+			if (!doblock(fs, state, PED_LE32_TO_CPU(uptr[i]), blk,
+				     i<<2, 0))
 				return 0;
 
 	if (!ext2_brelse(bh, 0))
@@ -216,7 +217,8 @@ static int dodindblock(struct ext2_fs *fs,
 
 	for (i=0;i<(fs->blocksize >> 2);i++)
 		if (uptr[i])
-			if (!doindblock(fs, state, uptr[i], blk, i<<2))
+			if (!doindblock(fs, state, PED_LE32_TO_CPU(uptr[i]),
+					blk, i<<2))
 				return 0;
 
 	if (!ext2_brelse(bh, 0))
@@ -245,7 +247,8 @@ static int dotindblock(struct ext2_fs *fs,
 
 	for (i=0;i<(fs->blocksize >> 2);i++)
 		if (uptr[i])
-			if (!dodindblock(fs, state, uptr[i], blk, i<<2))
+			if (!dodindblock(fs, state, PED_LE32_TO_CPU(uptr[i]),
+					 blk, i<<2))
 				return 0;
 
 	if (!ext2_brelse(bh, 0))
@@ -263,8 +266,10 @@ static int doinode(struct ext2_fs *fs, struct ext2_block_relocator_state *state,
 {
 	struct ext2_inode buf;
 
-	ext2_read_inode(fs, inode, &buf);
-	if (buf.i_blocks)
+	if (!ext2_read_inode(fs, inode, &buf))
+		return 0;
+
+	if (EXT2_INODE_BLOCKS(buf))
 	{
 		blk_t blk;
 		int   i;
@@ -273,7 +278,7 @@ static int doinode(struct ext2_fs *fs, struct ext2_block_relocator_state *state,
 
 		inodeoffset = ext2_get_inode_offset(fs, inode, &inodeblock);
 		for (i=0;i<EXT2_NDIR_BLOCKS;i++)
-			if ((blk = buf.i_block[i]) != 0)
+			if ((blk = EXT2_INODE_BLOCK(buf, i)) != 0)
 				if (!doblock(fs,
 					     state,
 					     blk,
@@ -282,7 +287,7 @@ static int doinode(struct ext2_fs *fs, struct ext2_block_relocator_state *state,
 					     0))
 					return 0;
 
-		if ((blk = buf.i_block[EXT2_IND_BLOCK]) != 0)
+		if ((blk = EXT2_INODE_BLOCK(buf, EXT2_IND_BLOCK)) != 0)
 			if (!doindblock(fs,
 					state,
 					blk,
@@ -290,7 +295,7 @@ static int doinode(struct ext2_fs *fs, struct ext2_block_relocator_state *state,
 					inodeoffset + offsetof(struct ext2_inode, i_block[EXT2_IND_BLOCK])))
 				return 0;
 
-		if ((blk = buf.i_block[EXT2_DIND_BLOCK]) != 0)
+		if ((blk = EXT2_INODE_BLOCK(buf, EXT2_DIND_BLOCK)) != 0)
 			if (!dodindblock(fs,
 					 state,
 					 blk,
@@ -298,7 +303,7 @@ static int doinode(struct ext2_fs *fs, struct ext2_block_relocator_state *state,
 					 inodeoffset + offsetof(struct ext2_inode, i_block[EXT2_DIND_BLOCK])))
 				return 0;
 
-		if ((blk = buf.i_block[EXT2_TIND_BLOCK]) != 0)
+		if ((blk = EXT2_INODE_BLOCK(buf, EXT2_TIND_BLOCK)) != 0)
 			if (!dotindblock(fs,
 					 state,
 					 blk,
@@ -323,7 +328,7 @@ static int doscan(struct ext2_fs *fs, struct ext2_block_relocator_state *state)
 	for (i=0;i<fs->numgroups;i++)
 	{
 		struct ext2_buffer_head *bh;
-		int			 j;
+		unsigned int		 j;
 		int			 offset;
 
 		if (fs->opt_verbose)
@@ -332,12 +337,12 @@ static int doscan(struct ext2_fs *fs, struct ext2_block_relocator_state *state)
 			fflush(stderr);
 		}
 
-		bh = ext2_bread(fs, fs->gd[i].bg_inode_bitmap);
+		bh = ext2_bread(fs, EXT2_GROUP_INODE_BITMAP(fs->gd[i]));
 		if (!bh)
 			return 0;
-		offset = i * fs->sb.s_inodes_per_group + 1;
+		offset = i * EXT2_SUPER_INODES_PER_GROUP(fs->sb) + 1;
 
-		for (j=0;j<fs->sb.s_inodes_per_group;j++)
+		for (j=0;j<EXT2_SUPER_INODES_PER_GROUP(fs->sb);j++)
 			if (bh->data[j>>3] & _bitmap[j&7])
 			{
 				if (!doinode(fs, state, offset + j))
@@ -397,7 +402,7 @@ static int ext2_block_relocator_copy(struct ext2_fs *fs, struct ext2_block_reloc
 		ptr = state->block;
 		while (numleft)
 		{
-			num = min(numleft, MAXCONT);
+			num = PED_MIN(numleft, MAXCONT);
 			while (num != 1)
 			{
 				if (ptr[0].num + num-1 == ptr[num-1].num &&
@@ -439,7 +444,7 @@ static int ext2_block_relocator_copy(struct ext2_fs *fs, struct ext2_block_reloc
 	}
 	else
 	{
-		int i;
+		blk_t i;
 
 		ped_exception_catch();
 		ped_exception_leave_all();
@@ -481,8 +486,8 @@ static int ext2_block_relocator_ref(struct ext2_fs *fs, struct ext2_block_reloca
 
 	if (fs->opt_debug)
 	{
-		if ((*((__u32 *)(bh->data + block->refoffset))) != block->num)
-		{
+		if (PED_LE32_TO_CPU(*((__u32 *)(bh->data + block->refoffset)))
+				!= block->num) {
 			fprintf(stderr,
 				"block %i ref error! (->%i {%i, %i})\n",
 				block->num,
@@ -499,7 +504,8 @@ static int ext2_block_relocator_ref(struct ext2_fs *fs, struct ext2_block_reloca
 		}
 	}
 
-	*((__u32 *)(bh->data + block->refoffset)) = block->dest;
+	*((__u32 *)(bh->data + block->refoffset))
+		= PED_LE32_TO_CPU(block->dest);
 	bh->dirty = 1;
 	ext2_brelse(bh, 0);
 
@@ -526,21 +532,24 @@ static int ext2_block_relocator_ref(struct ext2_fs *fs, struct ext2_block_reloca
 static int ext2_block_relocator_grab_blocks(struct ext2_fs *fs, struct ext2_block_relocator_state *state)
 {
 	int i;
-	int ptr;
+	blk_t ptr;
 
 	ptr = 0;
 
 	for (i=0;i<fs->numgroups;i++)
-		if (fs->gd[i].bg_free_blocks_count)
+		if (EXT2_GROUP_FREE_BLOCKS_COUNT(fs->gd[i]))
 		{
 			struct ext2_buffer_head *bh;
-			int j;
+			unsigned int j;
 			int offset;
 
-			bh = ext2_bread(fs, fs->gd[i].bg_block_bitmap);
-			offset = i * fs->sb.s_blocks_per_group + fs->sb.s_first_data_block;
+			bh = ext2_bread(fs, EXT2_GROUP_BLOCK_BITMAP(fs->gd[i]));
+			offset = i * EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb)
+				 + EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb);
 
-			for (j=state->newallocoffset;j<fs->sb.s_blocks_per_group;j++)
+			for (j=state->newallocoffset;
+			     j<EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb);
+			     j++)
 				if (!(bh->data[j>>3] & _bitmap[j&7]))
 				{
 					state->block[ptr++].dest = offset + j;
@@ -572,7 +581,7 @@ static int ext2_block_relocator_flush(struct ext2_fs *fs, struct ext2_block_relo
 	{
 	again:
 
-		for (i=0;i<state->usedentries-1;i++)
+		for (i=0; (unsigned int) i < state->usedentries-1; i++)
 			if (state->block[i].num >= state->block[i+1].num)
 			{
 				fprintf(stderr,
@@ -685,8 +694,10 @@ static int ext2_block_relocate_grow(struct ext2_fs *fs, struct ext2_block_reloca
 	blk_t newitoffset;
 	int   i;
 
-	newgdblocks = howmany(newsize - fs->sb.s_first_data_block, fs->sb.s_blocks_per_group);
-	newgdblocks = howmany(newgdblocks * sizeof(struct ext2_group_desc), fs->blocksize);
+	newgdblocks = howmany(newsize - EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb),
+			      EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb));
+	newgdblocks = howmany(newgdblocks * sizeof(struct ext2_group_desc),
+			      fs->blocksize);
 	if (newgdblocks == fs->gdblocks)
 		return 1;
 
@@ -701,15 +712,19 @@ static int ext2_block_relocate_grow(struct ext2_fs *fs, struct ext2_block_reloca
 		blk_t			 start;
 		int			 sparse;
 
-		bh = ext2_bread(fs, fs->gd[i].bg_block_bitmap);
-		start = (i * fs->sb.s_blocks_per_group) + fs->sb.s_first_data_block;
+		bh = ext2_bread(fs, EXT2_GROUP_BLOCK_BITMAP(fs->gd[i]));
+		start = (i * EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb))
+			+ EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb);
 		sparse = ext2_is_group_sparse(fs, i);
 
-		if (fs->gd[i].bg_inode_table < start + newitoffset
-		    || (sparse && ((fs->gd[i].bg_block_bitmap < start + newitoffset - 2) ||
-				   (fs->gd[i].bg_inode_bitmap < start + newitoffset - 1))))
+		if (EXT2_GROUP_INODE_TABLE(fs->gd[i]) < start + newitoffset
+		    || (sparse && ((EXT2_GROUP_BLOCK_BITMAP(fs->gd[i])
+						< start + newitoffset - 2)
+			       || (EXT2_GROUP_INODE_BITMAP(fs->gd[i])
+						< start + newitoffset - 1))))
 		{
-			diff = newitoffset - (fs->gd[i].bg_inode_table - start);
+			diff = newitoffset - (EXT2_GROUP_INODE_TABLE(fs->gd[i])
+					      - start);
 
 			for (j=0;j<diff;j++)
 			{
@@ -717,7 +732,8 @@ static int ext2_block_relocate_grow(struct ext2_fs *fs, struct ext2_block_reloca
 
 				k = fs->itoffset + fs->inodeblocks + j;
 				if (bh->data[k>>3] & _bitmap[k&7])
-					if (!ext2_block_relocator_mark(fs, state, start + k))
+					if (!ext2_block_relocator_mark(fs,
+							    state, start + k))
 					{
 						ext2_brelse(bh, 0);
 						return 0;
@@ -739,7 +755,8 @@ static int ext2_block_relocate_shrink(struct ext2_fs *fs, struct ext2_block_relo
 	int diff;
 	int i;
 
-	diff = howmany(newsize - fs->sb.s_first_data_block, fs->sb.s_blocks_per_group);
+	diff = howmany(newsize - EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb),
+		       EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb));
 	diff = howmany(diff * sizeof(struct ext2_group_desc), fs->blocksize);
 	diff = fs->gdblocks - diff;
 
@@ -755,13 +772,14 @@ static int ext2_block_relocate_shrink(struct ext2_fs *fs, struct ext2_block_relo
 		blk_t			 start;
 		int			 type;
 
-		offset = i * fs->sb.s_blocks_per_group + fs->sb.s_first_data_block;
+		offset = i * EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb)
+			 + EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb);
 		sparse = ext2_is_group_sparse(fs, i);
 
-		if (newsize >= offset + fs->sb.s_blocks_per_group)
+		if (newsize >= offset + EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb))
 			continue;		/* group will survive */
 
-		bh = ext2_bread(fs, fs->gd[i].bg_block_bitmap);
+		bh = ext2_bread(fs, EXT2_GROUP_BLOCK_BITMAP(fs->gd[i]));
 
 		if (newsize <= offset)
 			type = 2;		/* group is fully chopped off */
@@ -770,7 +788,9 @@ static int ext2_block_relocate_shrink(struct ext2_fs *fs, struct ext2_block_relo
 
 		if (!sparse && type == 2)
 		{
-			for (j=fs->gd[i].bg_inode_bitmap+1;j<fs->gd[i].bg_inode_table;j++)
+			for (j=EXT2_GROUP_INODE_BITMAP(fs->gd[i])+1;
+			     j<EXT2_GROUP_INODE_TABLE(fs->gd[i]);
+			     j++)
 			{
 				blk_t k;
 
@@ -786,17 +806,19 @@ static int ext2_block_relocate_shrink(struct ext2_fs *fs, struct ext2_block_relo
 
 		start = newsize;
 		if (type == 2)
-			start = fs->gd[i].bg_inode_table + fs->inodeblocks;
+			start = EXT2_GROUP_INODE_TABLE(fs->gd[i])
+				+ fs->inodeblocks;
 
 		start -= offset;
 
-		groupsize = fs->sb.s_blocks_per_group;
-		if (offset + groupsize > fs->sb.s_blocks_count)
-			groupsize = fs->sb.s_blocks_count - offset;
+		groupsize = EXT2_SUPER_BLOCKS_PER_GROUP(fs->sb);
+		if (offset + groupsize > EXT2_SUPER_BLOCKS_COUNT(fs->sb))
+			groupsize = EXT2_SUPER_BLOCKS_COUNT(fs->sb) - offset;
 
 		for (j=start;j<groupsize;j++)
 			if (bh->data[j>>3] & _bitmap[j&7])
-				if (!ext2_block_relocator_mark(fs, state, offset + j))
+				if (!ext2_block_relocator_mark(fs, state,
+							       offset + j))
 				{
 					ext2_brelse(bh, 0);
 					return 0;
@@ -822,7 +844,7 @@ int ext2_block_relocate(struct ext2_fs *fs, blk_t newsize)
 	state.resolvedentries = 0;
 	state.block = (struct ext2_block_entry *)fs->relocator_pool;
 
-	if (newsize < fs->sb.s_blocks_count)
+	if (newsize < EXT2_SUPER_BLOCKS_COUNT(fs->sb))
 		return ext2_block_relocate_shrink(fs, &state, newsize);
 
 	return ext2_block_relocate_grow(fs, &state, newsize);
