@@ -389,11 +389,11 @@ static int ext2_inode_relocator_ref(struct ext2_fs *fs, struct ext2_inode_reloca
 				{
 					fprintf(stderr,
 						"inode %li ref error! (->%li, [%i]={%i, %i})\n",
-						entry->num,
-						entry->dest,
+						(long) entry->num,
+						(long) entry->dest,
 						j,
 						entry->ref[j].block,
-						entry->ref[j].offset);
+						(int) entry->ref[j].offset);
 					ext2_brelse(bh, 0);
 
 					if (numerrors++ < 4)
