@@ -24,6 +24,7 @@ static const char _ext2_mkfs_c[] = "$Id: ext2_mkfs.c,v 1.4 1999/10/10 07:57:55 b
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <sys/stat.h>
 #include <time.h>
 #include <unistd.h>
@@ -249,6 +250,30 @@ error:
 	return 0;
 }
 
+/* returns the offset into the buffer of the start of the next dir entry */
+static int _set_dirent(void* buf, int offset, int block_size, int is_last,
+		       __u32 inode, char* name, int file_type)
+{
+	struct ext2_dir_entry_2 *dirent = (void*) (((char*)buf) + offset);
+	int name_len = strlen(name);
+	int rec_len;
+
+	if (is_last)
+		rec_len = block_size - offset;
+	else
+		rec_len = ped_round_up_to(name_len + 1 + 8, 4);
+
+	memset (dirent, 0, rec_len);
+
+	dirent->inode = PED_CPU_TO_LE32(inode);
+	dirent->name_len = name_len;
+	dirent->rec_len = PED_CPU_TO_LE16(rec_len);
+	dirent->file_type = file_type;
+	strcpy(dirent->name, name);
+
+	return offset + rec_len;
+}
+
 static int ext2_mkfs_create_lost_and_found_inode(struct ext2_fs *fs)
 {
 	struct ext2_buffer_head *bh;
@@ -256,6 +281,7 @@ static int ext2_mkfs_create_lost_and_found_inode(struct ext2_fs *fs)
 	__u32 data[fs->blocksize / 4];
 	int i;
 	struct ext2_inode inode;
+	int offset;
 
 	for (i=0;i<12;i++)
 	{
@@ -266,20 +292,20 @@ static int ext2_mkfs_create_lost_and_found_inode(struct ext2_fs *fs)
 			return 0;
 	}
 
-	memset(data, 0, fs->blocksize);
-	data[0] = PED_CPU_TO_LE32(11);
-	data[1] = PED_CPU_TO_LE32((1 << 16) | 12);
-	data[2] = PED_CPU_TO_LE32(0x2e);
-	data[3] = PED_CPU_TO_LE32(2);
-	data[4] = PED_CPU_TO_LE32((2 << 16) | (fs->blocksize - 12));
-	data[5] = PED_CPU_TO_LE32(0x2e2e);
+	/* create the directory entries, preallocating lots of blocks */
+	/* first block contains . and .. */
 	bh = ext2_bcreate(fs, blocks[0]);
 	if (!bh)
 		return 0;
-	memcpy(bh->data, data, fs->blocksize);
+	memset(bh->data, 0, fs->blocksize);
+	offset = _set_dirent(bh->data, 0, fs->blocksize, 0,
+			     11, ".", EXT2_FT_DIR);
+	offset = _set_dirent(bh->data, offset, fs->blocksize, 1,
+			     EXT2_ROOT_INO, "..", EXT2_FT_DIR);
 	bh->dirty = 1;
 	ext2_brelse(bh, 1);
 
+	/* subsequent blocks are empty */
 	memset(data, 0, fs->blocksize);
 	data[0] = 0;
 	data[1] = PED_CPU_TO_LE32(fs->blocksize);
@@ -291,6 +317,7 @@ static int ext2_mkfs_create_lost_and_found_inode(struct ext2_fs *fs)
 		ext2_brelse(bh, 1);
 	}
 
+	/* create inode */
 	memset(&inode, 0, sizeof(struct ext2_inode));
 	inode.i_mode = PED_CPU_TO_LE16(S_IFDIR | 0755);
 	inode.i_uid = 0;
@@ -321,28 +348,28 @@ static int ext2_mkfs_create_root_inode(struct ext2_fs *fs)
 	blk_t block;
 	__u32 data[fs->blocksize / 4];
 	struct ext2_inode inode;
+	int offset;
 
 	if (!(block = ext2_find_free_block(fs)))
 		return 0;
 	if (!ext2_set_block_state(fs, block, 1, 1))
 		return 0;
 
+	/* create directory entries */
 	memset(data, 0, fs->blocksize);
-	data[0]  = PED_CPU_TO_LE32(2);
-	data[1]  = PED_CPU_TO_LE32((1 << 16) | 12);
-	data[2]  = PED_CPU_TO_LE32(0x2e);
-	data[3]  = PED_CPU_TO_LE32(2);
-	data[4]  = PED_CPU_TO_LE32((2 << 16) | 12);
-	data[5]  = PED_CPU_TO_LE32(0x2e2e);
-	data[6]  = PED_CPU_TO_LE32(11);
-	data[7]  = PED_CPU_TO_LE32((10 << 16) | (fs->blocksize - 24));
-	strcpy (&data [8], "lost+found");
+	offset = _set_dirent(data, 0, fs->blocksize, 0,
+			     EXT2_ROOT_INO, ".", EXT2_FT_DIR);
+	offset = _set_dirent(data, offset, fs->blocksize, 0,
+			     EXT2_ROOT_INO, "..", EXT2_FT_DIR);
+	offset = _set_dirent(data, offset, fs->blocksize, 1,
+			     11, "lost+found", EXT2_FT_DIR);
 	bh = ext2_bcreate(fs, block);
 	memcpy(bh->data, data, fs->blocksize);
 	bh->dirty = 1;
 	if (!ext2_brelse(bh, 1))
 		return 0;
 
+	/* create inode */
 	memset(&inode, 0, sizeof(struct ext2_inode));
 	inode.i_mode = PED_CPU_TO_LE16(S_IFDIR | 0755);
 	inode.i_uid = 0;
@@ -367,7 +394,6 @@ static int ext2_mkfs_create_root_inode(struct ext2_fs *fs)
 }
 
 static
-
 int ext2_reserve_inodes(struct ext2_fs *fs)
 {
 	int i;
@@ -384,6 +410,9 @@ static int ext2_mkfs_init_sb (struct ext2_super_block *sb, blk_t numblocks,
 			      int inodes_per_group, int sparse_sb,
 			      int reserved_block_percentage)
 {
+	/* catch a bug in gcc 2.95.2 */
+	PED_ASSERT(numgroups != 0, return 0);
+
 	memset(sb, 0, 1024);
 
 	sb->s_inodes_count = PED_CPU_TO_LE32(numgroups * inodes_per_group);
@@ -393,7 +422,7 @@ static int ext2_mkfs_init_sb (struct ext2_super_block *sb, blk_t numblocks,
 
 	/* hack: this get's inc'd as we go through each group in
 	 * ext2_mkfs_write_meta()
-	 */
+	*/
 	sb->s_free_blocks_count = 0;
 	sb->s_free_inodes_count = PED_CPU_TO_LE32 (numgroups
 							* inodes_per_group);
@@ -409,15 +438,15 @@ static int ext2_mkfs_init_sb (struct ext2_super_block *sb, blk_t numblocks,
 	sb->s_max_mnt_count = PED_CPU_TO_LE16(30);
 	sb->s_magic = PED_CPU_TO_LE16(0xEF53);
 	sb->s_state = PED_CPU_TO_LE16(EXT2_VALID_FS);
-	sb->s_errors = 0;
-	sb->s_minor_rev_level = PED_CPU_TO_LE16(1);
+	sb->s_errors = PED_CPU_TO_LE16(EXT2_ERRORS_DEFAULT);
+	sb->s_minor_rev_level = 0;
 	sb->s_lastcheck = 0;
 	sb->s_checkinterval = 0;
 	sb->s_creator_os = 0;
-	sb->s_rev_level = 0;
+	sb->s_rev_level = PED_CPU_TO_LE16(1);
 	sb->s_def_resuid = 0;
 	sb->s_def_resgid = 0;
-	sb->s_first_ino = PED_CPU_TO_LE32(12);
+	sb->s_first_ino = PED_CPU_TO_LE32(11);
 	sb->s_inode_size = PED_CPU_TO_LE16(128);
 	sb->s_block_group_nr = 0;
 	sb->s_feature_compat = 0;
@@ -427,7 +456,7 @@ static int ext2_mkfs_init_sb (struct ext2_super_block *sb, blk_t numblocks,
 		sb->s_feature_ro_compat
 			|= PED_CPU_TO_LE32(EXT2_FEATURE_RO_COMPAT_SPARSE_SUPER);
 
-/* FIXME: let the user decide? */
+/* FIXME: let the user decide? _set_dirent() assumes FILETYPE */
 	sb->s_feature_incompat
 		|= PED_CPU_TO_LE32(EXT2_FEATURE_INCOMPAT_FILETYPE);
 
@@ -459,22 +488,22 @@ struct ext2_fs *ext2_mkfs(struct ext2_dev_handle *handle,
 	int last_group_blocks;
 	int last_group_admin;
 
-	if (log_block_size == -1)
+	if (log_block_size == 0)
 		log_block_size = 12;
 
 	handle->ops->set_blocksize(handle->cookie, log_block_size);
 
-	if (numblocks == -1)
+	if (numblocks == 0)
 		numblocks = handle->ops->get_size(handle->cookie);
 
-	if (blocks_per_group == (unsigned int) -1)
+	if (blocks_per_group == 0)
 		blocks_per_group = 8 << log_block_size;
 
 	first_block = (log_block_size == 10) ? 1 : 0;
 
 	numgroups = howmany(numblocks - first_block, blocks_per_group);
 
-	if (inodes_per_group == -1)
+	if (inodes_per_group == 0)
 		inodes_per_group = ped_round_up_to (
 			numblocks / numgroups / 2,
 			(1 << log_block_size) / sizeof(struct ext2_inode));
@@ -486,6 +515,7 @@ struct ext2_fs *ext2_mkfs(struct ext2_dev_handle *handle,
 		reserved_block_percentage = 5;
 
 	last_group_blocks = (numblocks - first_block) % blocks_per_group;
+	if (!last_group_blocks) last_group_blocks = blocks_per_group;
 	non_sparse_admin = 2
 			   + inodes_per_group * sizeof(struct ext2_inode)
 			   	/ (1 << log_block_size);
