@@ -36,7 +36,7 @@ static int ext2_add_group(struct ext2_fs *fs, blk_t groupsize)
 	if (fs->opt_verbose)
 		fprintf(stderr, "ext2_add_group\n");
 
-	if (!ped_realloc (&fs->gd,
+	if (!ped_realloc ((void*) &fs->gd,
 			  (fs->numgroups+1) * sizeof(struct ext2_group_desc)
 			      + fs->blocksize))
 		return 0;
@@ -318,7 +318,7 @@ static int ext2_del_group(struct ext2_fs *fs)
 	if (fs->opt_safe)
 		ext2_sync(fs);
 
-	ped_realloc (&fs->gd,
+	ped_realloc ((void*) &fs->gd,
 		     fs->numgroups * sizeof(struct ext2_group_desc)
 			      + fs->blocksize);
 
