@@ -18,6 +18,8 @@
 */
 
 #include <string.h>
+#include <uuid/uuid.h>
+
 #include <parted/disk_dos.h>
 #include <parted/disk_loop.h>
 #include <parted/disk_pc98.h>
@@ -264,6 +266,18 @@ fat_root_dir_clear (PedFileSystem* fs)
 				   fs_info->root_dir_sector_count);
 }
 
+/* hack: use the ext2 uuid library to generate a reasonably random (hopefully
+ * with /dev/random) number.  Unfortunately, we can only use 4 bytes of it
+ */
+static __u32
+_gen_new_serial_number ()
+{
+	uuid_t		uuid;
+
+	uuid_generate (uuid);
+	return * (__u32*) &uuid [0];
+}
+
 PedFileSystem*
 fat_create (PedGeometry* geom)
 {
@@ -420,6 +434,8 @@ fat_create (PedGeometry* geom)
 			return 0;
 	}
 
+	fs_info->serial_number = _gen_new_serial_number ();
+
 	if (!fat_boot_sector_set_boot_code (&fs_info->boot_sector))
 		goto error_free_buffers;
 	if (!fat_boot_sector_generate (&fs_info->boot_sector, fs))
