@@ -599,16 +599,27 @@ do_move (PedDevice** dev)
 		goto error_close_disk;
 	}
 	start = get_sector ();
-	if (!is_sector ()) {
-		help_on ("move");
-		goto error_close_disk;
+
+	if (peek_word ()) {
+		/* something specified for END */
+		if (!is_sector ()) {
+			help_on ("move");
+			goto error_close_disk;
+		}
+		end = get_sector ();
+		constraint = ped_constraint_any (disk);	/* FIXME */
+		if (!constraint)
+			goto error_close_disk;
+	} else {
+		/* no END specified; auto-compute */
+		end = start + part->geom.length - 1;
+		constraint = ped_constraint_any (disk);
+		if (!constraint)
+			goto error_close_disk;
+		constraint->min_size = part->geom.length; /* FIXME */
 	}
-	end = get_sector ();
 
 	old_geom = part->geom;
-	constraint = ped_constraint_any (disk);
-	if (!constraint)
-		goto error_close_disk;
 	if (!ped_disk_set_partition_geom (disk, part, constraint, start, end)) {
 		ped_constraint_destroy (constraint);
 		goto error_close_disk;
@@ -1096,7 +1107,7 @@ NULL),
 		str_list_create_unique ("move", _("move"), NULL),
 		do_move,
 		str_list_create (
-_("move MINOR START END          move partition MINOR"),
+_("move MINOR START [END]          move partition MINOR"),
 NULL),
 		str_list_create (_(minor_msg), _(start_end_msg), NULL)));
 
