@@ -37,11 +37,21 @@
 
 #ifdef HAVE_TERMCAP_H
 #include <termcap.h>
+#else
+extern int tgetnum (char* key);
 #endif
 
 #include <readline/readline.h>
 #include <readline/history.h>
 
+#ifndef HAVE_RL_COMPLETION_MATCHES
+#define rl_completion_matches completion_matches
+#endif
+
+#ifndef rl_compentry_func_t
+#define rl_compentry_func_t void
+#endif
+
 #endif /* HAVE_LIBREADLINE */
 
 #include <ctype.h>
@@ -59,9 +69,7 @@ static char* banner_msg = N_(
 "This program is distributed in the hope that it will be useful, "
 "but WITHOUT ANY WARRANTY; without even the implied warranty of "
 "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
-"GNU General Public License for more details.\n"
-"\n"
-"GNU Parted's development is supported by Conectiva, www.conectiva.com\n");
+"GNU General Public License for more details.\n");
 
 static char* usage_msg = N_(
 "Usage: parted [OPTION]... [DEVICE [COMMAND [PARAMETERS]...]...]\n"
@@ -314,9 +322,14 @@ char**
 complete_function (char* text, int start, int end)
 {
 	if (current_exception_opt)
-		return completion_matches (text, exception_command_generator);
+		return rl_completion_matches (
+				text,
+				(rl_compentry_func_t*)
+					exception_command_generator);
 	else
-		return completion_matches (text, command_generator);
+		return rl_completion_matches (
+				text,
+				(rl_compentry_func_t*) command_generator);
 }
 #endif /* HAVE_LIBREADLINE */
 
@@ -413,7 +426,7 @@ void
 parse_line (char* line)
 {
 	int	quoted = 0;
-	char	quote_char;
+	char	quote_char = 0;	/* shut gcc up! */
 	char	this_word [256];
 	int	i;
 
